=head1 NAME Pkg.pm

Pkg.pm - The Pkg module is the main interface for control scripts in the pkg
project. This module creates (and destroys) instances of the PkgConfig, and
PkgSystem objects and provides these objects to the script using this
object. The pkg object also provides some other useful methods such as command
line parsing.

$Id: Pkg.pm,v 1.39 2000/01/04 15:52:31 gomberg Exp $

=cut
my($VERSION) = "PKG " . "$Revision: 1.39 $foo";

package    Pkg;
use        Getopt::Long;
use        FileHandle;
use        PkgDebug;
use        PkgConfig;
use        PkgSystem;
use        strict;

########################################################################

=head1 Class fields

The Pkg object maintains internal references to other objects
as class fields (PKG_CONFIG_REF and PKG_SYSTEM_REF). Also a
refrence to the argument hash generated from the handle_args
method is stored in the ARGS field. Note that all fields are
considered private and should be accessed through the
appropriate method.

=cut

my %fields = (
	      PKG_CONFIG_REF        => undef,
	      PKG_SYSTEM_REF        => undef,
              ARGS                  => undef,             
	     );

my %private = (
	       GLOBAL_ONLY_FLAGS    => undef,
	       DONT_DETERMINE_ARGS  => undef,
	      );

########################################################################

=head1 Public Instance Methods

=head2 new()

    IN: No input parameters are required, (call like "Pkg->new()" ).
   OUT: Returns a Pkg object.
EFFECT: Creates and blesses a Pkg object and initializes class
        fields.

=cut

sub new {
    my $type = shift;
    my $class = ref( $type ) || $type;
    my $self = {};

    $self->{'fields'} = \%fields;
    $self->{'private'} = \%private;
    bless $self, $class;

    unless ( $self->{'fields'}->{'PKG_CONFIG_REF'} = PkgConfig->new() ) {
	PkgDebug::error( "Can't create a PkgConfig object" );
    }

    unless ( $self->{'fields'}->{'PKG_SYSTEM_REF'} =
	     PkgSystem->new( $self->{'fields'}->{'PKG_CONFIG_REF'} ) ) {
	PkgDebug::error( "Can't create a PkgSystem object" );
    }

    $self->_initialize();

    return $self;
}

#########################################################################

=head2 version()

    IN: 
   OUT: package version number
EFFECT: return version
       
=cut

sub version {
    my($self) = shift;

    return $VERSION;
}

#########################################################################

=head2 config()

    IN: multiple strings that set internal variables possible values:
        "global_only_flags" - specifies that the calling script only uses
        global option flags and does not define any of its own. 
        "dont_determine_args" - specifies that the calling script will use
        its own intelligence to fill in missing arguments. By default the Pkg
        object will do this work when handling arguments. 
   OUT: nothing
EFFECT: set state variable inside of global package object that determine
        further processing. 

=cut

sub config {
    my $self = shift;
    my(@args) = @_;

    my($arg);

    foreach $arg (@args) {
	$self->{'private'}->{'GLOBAL_ONLY_FLAGS'} = 1
	  if ($arg eq "global_only_flags");
	$self->{'private'}->{'DONT_DETERMINE_ARGS'} = 1
	  if ($arg eq "dont_determine_args");
    }
}

########################################################################

=head2 handle_args()

    IN: No input arguments (reads @ARGV).
   OUT: An argument hash. The argument hash has this form:
         arg{arch}     - the architecture string (-a/-arch)
         arg{pkg}      - the package name (-p/-pkg)
         arg{coll}     - the collection name (-c/-coll)
         arg{help}     - set if help flag is issued (-h/-u/-help)
         arg{dryrun}   - set if the dry run flag is given (-n/-dry)
         arg{debug}    - the debugging level (-d/-debug)
         arg{common}   - set if the install is common only (-com/-common)
         arg{src}      - path to the source (-s/-src)
         arg{args}     - an array refrence to the rest of the ARGV list
EFFECT: Sets the internal package variables and modifies other
        objects (such as setting debug levels or other fields).
        If an invalid argument is given, this method will print
        a warning message. Arguments not specified above will be
        passed through and put in arg{args}. Every script should
        first check the arg{help} flag. If it is set, there is
        no gaurentee that any of the other argument values are
        correct.

=cut

sub handle_args {
    my $self = shift;
    my $reconfig = undef;
    my %args = ( arch   => undef,
		 pkg    => undef,
		 coll   => undef,
		 help   => 0,
		 dryrun => 0,
		 debug  => 0,
		 common => undef,
		 src    => undef,
		 args   => undef
	       );


    Getopt::Long::config( "no_auto_abbrev",
			  "no_require_order",
			  "pass_through" );
    my $ret =
      GetOptions( 'a|arch=s',   \$args{'arch'},
		  'c|coll=s',   \$args{'coll'},
		  'p|pkg=s',    \$args{'pkg'},
		  'h|u|help',   \$args{'help'},
		  'n|dry',      \$args{'dryrun'},
		  'd|debug=i',  \$args{'debug'},
		  'com|common', \$args{'common'},
		  's|src=s',    \$args{'src'},
		  'cfg=s',      \$reconfig
		);

    unless( $ret ) {
	PkgDebug::warn( "Invalid command line arguments $ret" );
    }

    # Set the debug level
    PkgDebug::set_debug_level( $args{'debug'} );
    PkgDebug::debug( 6, "processing the command line" );

    # Set the dry run flag
    $self->{'fields'}->{'PKG_SYSTEM_REF'}->set_dry_run( $args{'dryrun'} );

    # Return if help
    if ( $args{'help'} ) {
	$self->{'fields'}->{'ARGS'} = \%args;
	return( $self->{'fields'}->{'ARGS'} );
    }

    # Use a different configuration file
    if ( $reconfig ) {
	$self->{'fields'}->{'PKG_CONFIG_REF'}->initialize( $reconfig );
    }
    
    my($file_env);
    $file_env = $self->_parse_env();
    # Determine common only, arch, coll, pkg
    if (!(defined ($self->{'private'}->{'DONT_DETERMINE_ARGS'}))) {
	$args{'common'} = $self->_determine_common_only( $args{'common'}, $file_env);
	# if a common only install may as well set the arch to common to be 
	# consistent
	if($args{'common'}){
	    $args{'arch'} = $self->{'fields'}->{'PKG_CONFIG_REF'}->COMMON();
	} else {
	    $args{'arch'} = $self->_determine_arch( $args{'arch'}, $file_env);
	}
	$args{'coll'} = $self->_determine_coll( $args{'coll'}, $file_env);
	$args{'pkg'} = $self->_determine_pkg( $args{'pkg'}, $file_env);

	# Find the source
	$args{'src'} = $self->_determine_src( $args{'src'}, $args{'coll'}, $args{'pkg'}, 
					     $file_env);
    }

    $args{'args'} = [ @ARGV ];

    $self->{'fields'}->{'ARGS'} = \%args;

    $self->_check_args() if($self->{'private'}->{'GLOBAL_ONLY_FLAGS'});    

    return( $self->{'fields'}->{'ARGS'} );
}

########################################################################

=head2 print_global_flags( $name )

    IN: The name of your script.
   OUT: The return value is not defined.
EFFECT: Prints all of the command line switches that are global
        to all scripts to STDOUT. This method is provided so that
        scripts can list the global command line flags when help
        is requested.

=cut

sub print_global_flags() {
    my( $self, $name ) = @_;

    $name = 'script' unless( $name );
    my(@versions);
    push(@versions, " " . $self->version() . "\n");
    push(@versions, $self->get_pkg_config()->version() . "\n");
    push(@versions, "Configuration file : " . 
	 $self->get_pkg_config()->CONFIG_LOC() . "\n");
    push(@versions, $self->get_pkg_system()->version() . "\n");

    print STDOUT <<"EOF"

Using the following configuration:

@versions

The following are global command line switches:

    -h or -help or -u : Request a help message.
    -a or -arch       : The architecture that $name should use.
    -c or -coll       : The collection name that $name should use.
    -p or -pkg        : The package name that $name should use.
    -n or -dry        : Print system calls but do not execute.
    -d or -debug      : The debugging level that $name should use.
    -com or -common   : Specify a common-only install.
    -s or -src        : Specify the path to the pkg source.
    -cfg config_file  : Specify the configuration file to use.

EOF

}

########################################################################

=head2 get_args()

    IN: There are no input arguments.
   OUT: A refrence to the argument hash as defined in handle_args(),
        or undef if handle_args has not been called.
EFFECT: No side effects.

=cut

sub get_args {
    my $self = shift;

    unless ( defined $self->{'fields'}->{'ARGS'} ) {
	PkgDebug::warn( "arguments are not defined" );
	return undef;
    }

    return( $self->{'fields'}->{'ARGS'} );
}

########################################################################

=head2 get_pkg_config()

     IN: There are no input arguments.
    OUT: The PkgConfig object.
EFFECTS: No side effects.

=cut

sub get_pkg_config {
    my $self = shift;

    PkgDebug::debug( 9, "Getting PkgConfig object" );
    return( $self->{'fields'}->{'PKG_CONFIG_REF'} );
}

########################################################################

=head2 get_pkg_system()

     IN: There are no input arguments.
    OUT: The PkgSystem object.
EFFECTS: No side effects.

=cut

sub get_pkg_system {
    my $self = shift;

    PkgDebug::debug( 9, "Getting PkgSystem object" );
    return( $self->{'fields'}->{'PKG_SYSTEM_REF'} );
}

########################################################################

=head2 common_only()

     IN: nothing
    OUT: returns 1 if common only install
EFFECTS: No side effects.

=cut

sub common_only {
    my($self) = @_;

    if ($self->get_args()->{common}) { 
	return 1;
    } else {
	return 0;
    }
}


########################################################################

=head2 get_template_install_path()

     IN: the name of the template
    OUT: full path to a template
EFFECTS: No side effects.

=cut

sub get_template_install_path {
    my ($self, $template) = @_;

    my ($template_hash_ref) = $self->get_pkg_config()->TEMPLATE_NAMES();

    my($file_name);
    if (defined($template_hash_ref->{$template})) {
	my($template_info) = $template_hash_ref->{$template};
	# determine where the template should go
      SWITCH: {
	  $ {$template_info}[1] eq "COMMON" && do {
	      $file_name = $self->get_common_package_dir();
	      last SWITCH;
	  };
	  $ {$template_info}[1] eq "ARCH" && do {
	      $file_name = $self->get_package_dir();
	      last SWITCH;
	  };
	  # error, the tree is invalid
	  return 0;
      }
	# construct full pathname 
	if( $ {$template_info}[2] ) {
	    $file_name .= "/" . $ {$template_info}[2] . "/" . $ {$template_info}[0];
	} else {
	    # not really an error, just indicates that this particular
	    # template shouldn't be copied into package subdirectories.
	    return 0;
	}
    } else {
	# error, invalid template name
	return 0;
    }

    return $file_name;
}

########################################################################

=head2 get_template_source_path()

     IN: the name of the template
    OUT: full path to the template source file
EFFECTS: No side effects.

=cut

sub get_template_source_path {
    my ($self, $template) = @_;

    my ($template_hash_ref) = $self->get_pkg_config()->TEMPLATE_NAMES();

    my($file_name) = $self->get_pkg_config()->TEMPLATE_BASE();

    if (defined($template_hash_ref->{$template})) {
	my($template_info) = $template_hash_ref->{$template};
	$file_name .= "/" . $ {$template_info}[0];
    } else {
	# error, invalid template name
	return 0;
    }

    return $file_name;    
}

########################################################################

=head2 get_template_filename()

     IN: the name of the template
    OUT: just the filename of the template in question
EFFECTS: No side effects.

=cut

sub get_template_filename {
    my ($self, $template) = @_;

    my ($template_hash_ref) = $self->get_pkg_config()->TEMPLATE_NAMES();

    my($file_name);
    if (defined($template_hash_ref->{$template})) {
	my($template_info) = $template_hash_ref->{$template};
	$file_name = $ {$template_info}[0];
    } else {
	# error, invalid template name
	return 0;
    }

    return $file_name;
}


########################################################################

=head2 get_package_dir()

    IN: There are no input arguments.
   OUT: <software>/<arch>/<collection>/packages/<package>
EFFECT: 

=cut

sub get_package_dir {
    my($self) = shift;

    unless ( defined $self->{'fields'}->{'ARGS'} ) {
        PkgDebug::warn( "cwd_package_dir: The argument hash is not defined" );
	return( 0 );
    } else {
	my( $software ) = $self->{'fields'}->{'PKG_CONFIG_REF'}->SOFTWARE_ROOT();
	my( $arg_ref ) = $self->get_args();
        PkgDebug::error( "arguments are not defined yet" ) unless( $arg_ref );
	my( $arch ) = $$arg_ref{'arch'};
	my( $coll ) = $$arg_ref{'coll'};
	my( $pkg ) = $$arg_ref{'pkg'};
	return "$software/$arch/$coll/packages/$pkg/";	
    }

    return 0;
}

########################################################################
# get_common_package_dir
########################################################################

=head2 get_common_package_dir()

    IN: There are no input arguments.
   OUT: <software>/<common>/<coll>/packages/<package>
EFFECT: 

=cut

sub get_common_package_dir {
    my($self) = shift;

    unless ( defined $self->{'fields'}->{'ARGS'} ) {
        PkgDebug::warn( "cwd_package_dir: The argument hash is not defined" );
	return( 0 );
    } else {
	my( $software ) = $self->{'fields'}->{'PKG_CONFIG_REF'}->SOFTWARE_ROOT();
	my( $arg_ref ) = $self->get_args();
        PkgDebug::error( "arguments are not defined yet" ) unless( $arg_ref );
	my( $arch ) = $self->{'fields'}->{'PKG_CONFIG_REF'}->COMMON();
	my( $coll ) = $$arg_ref{'coll'};
	my( $pkg ) = $$arg_ref{'pkg'};
	return "$software/$arch/$coll/packages/$pkg/";	
    }

    return 0;
}

########################################################################

=head2 get_www_filename()

    IN: ARCH, COMMON or nothing
   OUT: with ARCH or COMMON, returns the full pathname of the www 
        output files in either ARCH or COMMON. Called with nothing, 
        returns a refference to an array of both COMMON and ARCH specific filenames
EFFECT: No side effects.

=cut

sub get_www_filename {
    my($self, $tree) = @_;

    # setup the filenames
    my($pkg_cfg) = $self->get_pkg_config();
    my($a_index) = $self->get_package_dir();
    $a_index .= "/" . $pkg_cfg->PKG_WWW_DIR();
    $a_index .= "/" . $self->get_template_filename("AINDEX");
    my($c_index) = $self->get_common_package_dir();
    $c_index .= "/" . $pkg_cfg->PKG_WWW_DIR();
    $c_index .= "/" . $self->get_template_filename("CINDEX");
    
    if($tree){
      SWITCH: {	
	  $tree eq "ARCH" && do {
	      return $a_index;
	      last SWITCH;
	  };
	  $tree eq "COMMON" && do {
	      return $c_index;
	      last SWITCH;
	  }
      }
    } else {
	if ($self->common_only()){
	    return [$c_index];
	} else {
	    return [$a_index, $c_index];
	}
    }
}

########################################################################

=head2 verify_pkg()

    IN: nothing
   OUT: returns true if the package directories are present and 
        have subdirectories. False otherwise
EFFECT: No side effects.

=cut

sub verify_pkg {
    my($self) = shift;

    # these checks may be a little redundant for a 
    # a comon only install since package_dir and common_package_dir
    # might be the same

    if (-e ($self->get_package_dir()) &&
	-e ($self->get_common_package_dir())) {
	if($self->get_pkg_system()->get_subdirs($self->get_package_dir()) &&
	   $self->get_pkg_system()->get_subdirs($self->get_common_package_dir())) {
	    return 1;
	}
    }    

    return 0
}

########################################################################

=head2 cwd_package_dir()

    IN: There are no input arguments.
   OUT: True if the current working directory was changed, False otherwise.
EFFECT: Changes the current working directory of the script to
        <software>/<arch>/<collection>/packages/<package>.

=cut

sub cwd_package_dir {
    my( $self ) = shift;

    PkgDebug::debug( 3, "Trying to change cwd to package dir" );

    unless ( defined $self->{'fields'}->{'ARGS'} ) {
        PkgDebug::warn( "cwd_package_dir: The argument hash is not defined" );
	return( 0 );
    } else {
	if ( $self->{'fields'}->{'PKG_SYSTEM_REF'}->set_cwd( $self->get_package_dir()) ) {
	    return( 1 );
	}
    }

    return( 0 );
}

########################################################################

=head2 edit_in_line

    IN: filename of the file, and text to put at bottom
   OUT: true if edit succeeds
EFFECT: starts editor to fill in a file and temporarily 
        appends provided text at the bottom 

=cut

sub edit_in_line {
    my($self, $filename, $bottom_text) = @_;

    my($editor) = $ENV{EDITOR};
    
    $editor = "vi" if(!$editor);

    my($info_file) = new FileHandle;

    $info_file->open(">>$filename");
    
    print $info_file $bottom_text;

    $info_file->close();

    # Edit the info file
    system("$editor $filename");

    $info_file->open($filename);
    my(@buffer);

    my($skip) = 0;
    while (<$info_file>){
	if (/={79}/){
	    if($skip){
		$skip = 0;
		next;
	    } else {
		$skip = 1;
	    }
	}	
	
	push (@buffer, $_) if(!$skip);
    }

    $info_file->close();

    $info_file->open(">$filename");
    print $info_file(@buffer);
    $info_file->close();
}

########################################################################

=head2 is_element( $value, @array )

    IN: A string value to be checked and an array.
   OUT: True if the value is in the array, False otherwise.
EFFECT: No side effects.

=cut

sub is_element {
    my( $self, $value, @array ) = @_;

    foreach ( @array ) {
	return 1 if( $value eq $_ );
    }

    return 0;
}

########################################################################

=head2  find_collection()

     IN: reference to package object, package and architecture names
    OUT: collection name or 0 if not found
 EFFECT: checks all the collections in a particular architecture for a 
         specified package and returns the collection

=cut

sub find_collection {
    my($self, $package, $arch) = @_;

    my($collection);
    my($pkg_dir);

    my($coll_ref) = $self->{'fields'}->{'PKG_CONFIG_REF'}->COLLECTIONS();
    my($good_collection) = 0; 

    foreach $collection (@$coll_ref) {
	$pkg_dir = $self->{'fields'}->{'PKG_CONFIG_REF'}->SOFTWARE_ROOT()."/".
	  $arch."/".$collection."/packages/".$package;

	if (-d $pkg_dir) {
	    if ($good_collection) {
		PkgDebug::warn("Package $package exists in more then one collection : $good_collection, $collection");
		PkgDebug::warn("Returning only the first collection");
	    } else {	       
		$good_collection = $collection;
	    }
	} 
    }

    return $good_collection;
}

########################################################################
# rename_next
########################################################################

=head2 rename_next()

     IN: reference to package object, type of output file to rename
         [configure, make, make-install]
    OUT: 0 on error 1 on success
 EFFECT: renames a file like configure.out to configure.out.1 with
         the number increasing, for each sucessive file

=cut

sub rename_next {
    my( $self, $src, $filename) = @_;

    my($PKG_SYSTEM) = $self->get_pkg_system();
    my( @files ) = $PKG_SYSTEM->get_files( $src );
    unless( @files ) {
	PkgDebug::error( "Can't get files in the current directory" );
    }

    my( $next ) = 1;
    foreach ( @files ) {
	if ( /$filename\.(\d+)/ ) {
	    if ( $1 >= $next ) {
		$next = $1 + 1;
	    }
	}
    }

    unless( $PKG_SYSTEM->rename_file( "$src/$filename", "$src/$filename.$next" ) ) {
	PkgDebug::error( "Can't rename $src/$filename to $src/$filename.$next" );
    }
}

########################################################################
#
# PRIVATE METHODS
#
########################################################################

=head1 Private Class Methods

=head2 _initialize()

    IN: There are no input arguments.
   OUT: The return value is not defined.
EFFECT: Initializes class fields to default values.

=cut

sub _initialize {
    my $self = shift;
    my $type = ref( $self ) || $self;

    PkgDebug::debug( 1, "Creating and initializing $type object" );
}

########################################################################

=head2 _parse_env()

 IN: There are no input arguments
 OUT: The return value is not defined
 EFFECT: Parses the ~/.pkg file and defines environment variables that are
         used by _determine_common_only(), _determine_arch(), 
         _determine_coll, _determine_pkg, and _determine_src. These values
         are used by handle_args() and passed on to the calling script

=cut

sub _parse_env {
    my $self = shift;

    my($home) = $ENV{HOME};
    my(%file_env);

    if ( -f "$home/.pkg" ) {
	PkgDebug::debug( 1, "Setting environment variables from $home/.pkg" );
	open CONFIG, "$home/.pkg";

	##
	# This while loop parses the ~/.pkg file and sets up environment values
	# accordingly.
	#

	while ( <CONFIG> ) {
	    chop;
	    my( $var, $value ) = split /:/;
	    if ( !$file_env{$var} ) {
		$file_env{$var} = $value;
	    }
	}

	close CONFIG;
    }

    return \%file_env;
}

########################################################################

=head2 _determine_common_only( $common )

    IN: Optionally, the common only flag specified on the command line.
   OUT: True if this is a common only install, False otherwise.
EFFECT: Will check the environment variable to see if we have a
        common only install.

=cut

sub _determine_common_only {
    my( $self, $common, $file_env) = @_;

    PkgDebug::debug( 9, "Checking for common only install" );

    my($common_name) = $self->{'fields'}->{'PKG_CONFIG_REF'}->COMMON();
    # First look at the command line
    if ( $common ) {
	return( 1 );
    }  elsif ( defined $ENV{'PKG_ARCH'} && $ENV{'PKG_ARCH'} eq $common_name ) {
	print STDOUT "Assuming common only install from PKG_ARCH environment setting.\n";
	return( 1 );
    } elsif ($file_env->{PKG_ARCH} eq $common_name) {
	print STDOUT "Assuming common only install from PKG_ARCH .pkg setting.\n";
	return ( 1 );
    }

    return( 0 );
}

########################################################################

=head2 _determine_arch( $arch )

    IN: Optionally, the arch specified on the command line.
   OUT: The arch that should be used for this pkg instance.
EFFECT: Checks against valid arches in PkgConfig and will
        print a message if arch is determined from ENV
        variables or whatami script.

=cut

sub _determine_arch {
    my( $self, $arch, $file_env ) = @_;
    my $retarch;

    PkgDebug::debug( 9, "Determining current arch" );

    my $whatami = $self->{'fields'}->{'PKG_SYSTEM_REF'}->get_whatami();

    # First look at the command line
    if ( $arch ) {
	$retarch = $arch;
	if ( $whatami and $retarch ne $whatami ) {
	    PkgDebug::warn( "arch \'$retarch\' may be incorrect ($whatami)." );
	}
    }

    # Next look at the environment
    elsif ( $retarch = $ENV{'PKG_ARCH'} ) {
	print STDOUT "Using arch \'$retarch\' from PKG_ARCH environment setting.\n";
    }

    elsif ( $retarch = $file_env->{PKG_ARCH} ){
	print STDOUT "Using arch \'$retarch\' from PKG_ARCH .pkg setting.\n";
    }

    # Finally look at the whatami string
    elsif ( $whatami ) {
	$retarch = $whatami;
	print STDOUT "Using arch \'$retarch\' from \'whatami\' script.\n";
    } else {
	PkgDebug::error( "can't determine arch" );
    }

    # Check for a valid arch
    if ( defined $retarch ) {
	if (!$self->_check_valid_arch($retarch)) {
	    PkgDebug::error( "\'$retarch\' is not a supported arch" ); 
	}
    }

    if ( ($whatami and $retarch ne $whatami) and 
	$retarch ne $self->{'fields'}->{'PKG_CONFIG_REF'}->COMMON() ) {

        PkgDebug::warn( "arch \'$retarch\' may be incorrect ($whatami)." );
    }

    return $retarch;
}

########################################################################

=head2 _check_valid_arch( $arch )

    IN: architecture name 
   OUT: architecture name if supported, 0 if not supported
EFFECT: Checks against the valid architecture names and returns status

=cut

sub _check_valid_arch {
    my($self, $retarch) = @_;

    my($good_arch_ref);
    if (defined ($retarch)) {
	$good_arch_ref = $self->{'fields'}->{'PKG_CONFIG_REF'}->ARCHES();
	unless( $self->is_element( $retarch, @$good_arch_ref ) ) {    
	    return 0;
	}
	return $retarch;
    }
    
    return 0;
}

########################################################################

=head2 _determine_coll( $coll )

    IN: Optionally, the collection specified on the command line.
   OUT: The collection that should be used for this pkg instance.
EFFECT: Checks against valid collections in PkgConfig and will
        print a message if collection is determined from ENV
        variables or current directory.

=cut

sub _determine_coll {
    my( $self, $coll, $file_env ) = @_;
    my $retcoll;

    PkgDebug::debug( 9, "Determining current collection" );

    # Get the collection from the cwd
    my $good_coll_ref = $self->{'fields'}->{'PKG_CONFIG_REF'}->COLLECTIONS();
    my $colldir = $self->{'fields'}->{'PKG_SYSTEM_REF'}->get_cwd();

    my @colldirarr = split( /\/|\\/, $colldir );
    my $collect = undef;

    foreach $colldir ( reverse @colldirarr ) {
	if ( $self->is_element( $colldir, @$good_coll_ref ) ) {
	    $collect = $colldir;
	    last;
	}
    }

    # First look at the command line
    if ( $coll ) {
	$retcoll = $coll;
    }

    # Next look at environment variables
    elsif ( $retcoll = $ENV{'PKG_COLL'} ) {
	print STDOUT "Using collection \'$retcoll\' from PKG_COLL environment setting.\n";
    }

    elsif ( $retcoll = $file_env->{'PKG_COLL'} ) {
	print STDOUT "Using collection \'$retcoll\' from PKG_COLL .pkg setting.\n";
    }

    # Finally try cwd
    elsif ( $retcoll = $collect ) {
	print STDOUT "Using collection \'$retcoll\' from current working directory.\n";
    } else {
        PkgDebug::error( "Can't determine collection from current working directory
      or from the PKG_COLL environment variable." );
    }

    if ( $collect and $retcoll ne $collect ) {
        PkgDebug::warn( "collection \'$retcoll\' may be incorrect ($collect)." );
    }

    unless( $self->is_element( $retcoll, @$good_coll_ref ) ) {
        PkgDebug::error( "\'$retcoll\' is not a supported collection" );
    }

    return $retcoll;
}

########################################################################

=head2 _determine_pkg( $pkg )

    IN: Optionally, the pkg name specified on the command line.
   OUT: The pkg name that should be used for this pkg instance.
EFFECT: Will print a message if pkg name is determined from ENV
        variables or current directory.

=cut

sub _determine_pkg {
    my( $self, $pkg, $file_env ) = @_;
    my $retpkg;

    PkgDebug::debug( 9, "Determining current package" );

    my( $pkg_regx ) = $self->{'fields'}->{'PKG_CONFIG_REF'}->PKG_REGEXP();

    # Get the package from the cwd
    my $pkgdir = $self->{'fields'}->{'PKG_SYSTEM_REF'}->get_cwd();
    my @pkgdirarr = split( /\/|\\/, $pkgdir );
    $pkgdir = undef;
    # start from the end of the current path and search for
    # the PKG regular expression
    while ($_ = pop(@pkgdirarr)) {
	if ($_ =~ /$pkg_regx/) {
	    $pkgdir = $_;
	    last;
	}
    }

    # First look at the command line
    if ( $pkg ) {
	$retpkg = $pkg;
    }

    # Next look at environment variables
    elsif ( $retpkg = $ENV{'PKG'} ) {
	print STDOUT "Using package \'$retpkg\' from PKG environment setting.\n";
    }

    elsif ( $retpkg = $file_env->{PKG} ){
	print STDOUT "Using package \'$retpkg\' from PKG .pkg setting.\n";
    }				

    # Finally try the cwd
    elsif ( $retpkg = $pkgdir ) {
	print STDOUT "Using package \'$retpkg\' form current working directory.\n";
    } else {
	PkgDebug::error( "Can't determine package from current working directory
      or from the PKG environment variable." );
    }

    if ( $pkgdir and $retpkg ne $pkgdir ) {
        PkgDebug::warn( "package \'$retpkg\' may be incorrect ($pkgdir)." );
	PkgDebug::warn( "check for an old ~/.pkg file" );
    }

    # See if the package name is cool
    if ( $retpkg !~ /$pkg_regx/ ) {
        PkgDebug::error( "\'$retpkg\' is not a valid package name." );
    }

    return $retpkg;
}

########################################################################

=head2 _determine_src( $src )

    IN: Optionally, the source path 
        The collection 
        The package name
   OUT: The src path that should be used for this pkg instance.
EFFECT: Will print a message if src path is determined from ENV
        variables or default directory specified in pkg.cfg.

=cut

sub _determine_src {
    my( $self, $src, $coll, $pkg, $file_env ) = @_;
    my $retsrc;

    PkgDebug::debug( 9, "Determining package source" );

    # See if the source exists where it should
    my( $def_src ) = $self->{'fields'}->{'PKG_CONFIG_REF'}->SOURCE_BASE();
    my( $srcdir ) = "$def_src/$coll/$pkg";

    # if the default doesn't exist then don't use it 
    undef $srcdir unless( -d $srcdir );

    # First look at the command line
    if ( $src ) {
	$retsrc = $src;
    } 
    
    # Next look at environment variables
    elsif ( $retsrc = $ENV{'P_SRC'} ) {
	print STDOUT "Using source path \'$retsrc\' from P_SRC environment setting.\n";
    }

    elsif ( $retsrc = $file_env->{'P_SRC'} ){
	print STDOUT "Using source path \'$retsrc\' from P_SRC .pkg setting.\n";
    }

    # Next look at the default source location
    elsif ( defined $srcdir ) {
	$retsrc = $srcdir;
    }

    # Finally use the current directory
    else {
	$retsrc = $self->{'fields'}->{'PKG_SYSTEM_REF'}->get_cwd();
	print STDOUT "Using current directory \'$retsrc\' as source path.\n";
    }

    if ( defined $srcdir and ($retsrc ne $srcdir) ) {
        PkgDebug::warn( "source path \'$retsrc\' may be incorrect ($srcdir)." );
    }

    return $retsrc;
}

sub _check_args {
    my $self = shift;

    my $error_args = "";
    my $arg; 
    my $common_name;

    $common_name = $self->{'fields'}->{'PKG_CONFIG_REF'}->COMMON();
    # first expression is XOR of arch and common
    # then check for the rest of the required fields to be defined.
    if ( (($self->{'fields'}->{'ARGS'}->{'common'} xor 
	  $self->{'fields'}->{'ARGS'}->{'arch'}) ||
	  $self->{'fields'}->{'ARGS'}->{'arch'} eq $common_name)  &&
	 defined $self->{'fields'}->{'ARGS'}->{'pkg'} && 
	 defined $self->{'fields'}->{'ARGS'}->{'coll'} && 
	 defined $self->{'fields'}->{'ARGS'}->{'src'}        
       ) {
	return;
    } else {
	print_global_flags();
	foreach $arg (@{$self->{'fields'}->{'ARGS'}->{'args'}}) {
	    $error_args .= $arg . " ";
	}
	chop($error_args);
	PkgDebug::error("Invalid argument[s] < $error_args >");
    }				
}

########################################################################
