=head1 NAME PkgConfig.pm

PkgConfig.pm - The PkgConfig module parses the pkg.cfg file for global
definitions that the package system will use. PkgConfig should not be
instanced driectly by scripts. Instead it should be called from the Pkg
module.

$Id: PkgConfig.pm,v 1.46 2000/01/04 15:52:32 gomberg Exp $

=cut

my($VERSION) = "PkgConfig " . "$Revision: 1.46 $foo";

package       PkgConfig;
use           PkgDebug;

#########################################################################

=head1 Class Fields

The PkgConfig object has many fields which are used to store 
the variable information. Fields are accessed via the perl
AUTOLOAD function so that method and field names coincide.
The fields are defined as follows:

=over 4

=item *

CONFIG_LOC:        The fully qualified path to the pkg.cfg file.
                   Returns: a string.

=item *

SOFTWARE_ROOT:     The fully qualified path to the root of the
                   software tree.
                   Returns: a string.

SOFTWARE_LINK:     The root doesn't have to be a directory, it can
                   be a link into an nfs mount.

=item *

SOFTWARE_PERMS:    Permissions to give to the software tree.
                   Returns: a reference to an array.

=item *

SOFT_ROOT:         Name that the architecture dependent portion of
                   the software tree should be linked to.
                   Returns: a string.

=item *

ARCHES:            List of architectures that exist in the software tree
                   (includes the "common" architecture).
                   Returns: a reference to an array.

=item *

COMMON:            The name used to identify the common architecture.
                   Returns: a string.

=item *

ARCH_PERMS:        Permissions to give each architecture specific directory.
                   Returns: a reference to a hash which has architecture names
                            as keys and array references as key values.

=item *

ARCH_DEF_PERMS:    Default permissions for files and directories at the
                   architecture level.
                   Returns: a reference to an array.

=item *

ARCH_LINKS:        Links into nfs space if architectures are served by 
                   different physical servers. 

=item *

COLLECTIONS:       List of collections names.
                   Returns: a reference to an array.

=item *

COLL_PERMS:        Permissions to give to the collection directories.
                   Returns: a reference to a hash which has collection names
                            as keys and array references as key values.

=item *

COLLECT_IN_LOCAL:  The collections which should be linked into /usr/local,
                   and the order in which they should be linked.
                   Returns: a reference to an array of strings.
=item *

COLL_DEF_PERMS:    Default permissions for files and directories at the
                   collection level.
                   Returns: a reference to an array.

=item *

COLL_SUBDIRS:      The subdirectories that should be created under each collection.
                   Returns: a reference to an array.

=item *

PACKAGE_DIR:       The name of the directory where packages are located.
                   Retruns: a string.

=item *

MAN_DIR:           The name of the directory where manual pages are located.
                   Returns: a string.

=item *

COLL_SUBDIR_PERMS: The permissions to give the collection subdirectories.
                   Returns: a reference to a hash which has collection subdir
                            names as keys and array references as key values.

=item *

COLL_SUB_DEF_PERMS: The default permissions for files and directories at the
                    collection subdirectory level.
                    Returns: a reference to an array.

=item *

PKG_SUBDIRS:       The subdirectories to create under each package directory.
                   Returns: a reference to an array.

=item *

PKG_ADMIN_DIR:     The name of the directory where administrative file are located.
                   Returns: a string.

=item *

PKG_WWW_DIR:       The name of the directory where www files are located.
                   Returns: a string.

=item *

REQUIRED_DIRS:     The name of the directories to not delete even if they are
                   empty.
                   Returns: a reference to an array.

=item *

PKG_SUBDIR_PERMS:  The permissions to give the package subdirectories.
                   Returns: a reference to a hash which has package subdir names
                            as keys and array references as key values.

=item *

PKG_SUB_DEF_PERMS: The default permissions for files and directories at the
                   package level.
                   Returns: a reference to an array.

=item *

PKG_FILE_PERMS:    The permissions to give files copied into the package
                   subdirectories (non-executable package project files).
                   Returns: a refrence to an array.

=item *

MAN_SUBDIRS:       The man page categories to create.
                   Returns: a reference to an array.

=item *

COLL_MAN_PERMS:    The man directory permissions for each collection
                   Returns: a reference to a hash which has man dir names
                            as keys and array references as key values.

=item *

PKG_MAN_PERMS:     The man directory permissions for each package
                   Returns: a reference to a hash which has man dir names
                            as keys and array references as key values.

=item *


PKG_SRC_LINK:      The name that should be used to link the source.
                   Returns: a string.

=item *

PKG_COMMON_LINK:   The name that should be used to link the source.
                   Returns: a string.

=item *

TEMPLATE_NAMES:    The names of templates that should be installed when a
                   package is created and the location to install them to.
                   Returns: a reference to a hash where each value is a
                   reference to an array. The keys are template names which 
                   can be INFO, AINDEX, CINDEX and ARCH. 
                   The first value in the returned array is the filename of 
                   the template. The second value is either COMMON or ARCH, 
                   specifying the template to go into either the common
                   or architecture specific package directory. The last value
                   is the directory under package for the templates to 
                   go into. Usually this is whatever PKG_ADMIN_DIR is set to.

=item *

PKG_REGEXP:        The perl regular experssion that package names should
                   conform to.
                   Returns: a string.

=item *

TEMPLATE_BASE:     The base directory to look for templates in.
                   Returns: a string.

=item *

SOURCE_BASE:       The base directory to look for package source files in.
                   Returns: a string.

=item *
LOCAL_BASE:        The location of /usr/local.  
                   Returns: a string.

=item *

WHATAMI_PATH:      The path to the whatami executable.
                   Returns: a string.

=back

=cut

# Field Definitions. Initialize to default values.

my %fields = (
	      CONFIG_LOC           => 'PKG_CONFIG_LOC',
	      SOFTWARE_ROOT        => '/software',
              SOFTWARE_LINK        => undef,
	      SOFTWARE_PERMS       => undef,
	      SOFT_ROOT            => '/soft',
	      ARCHES               => undef,
              ARCH_LINKS           => undef,
	      COMMON               => 'common',
	      ARCH_PERMS           => undef,
              ARCH_DEF_PERMS       => undef,
	      COLLECTIONS          => undef,
	      COLL_PERMS           => undef,
              COLL_DEF_PERMS       => undef,
	      LOCAL_BASE           => '/usr/local',
	      COLLECT_IN_LOCAL     => undef,
	      COLL_SUBDIRS         => undef,
	      PACKAGE_DIR          => 'packages',
	      MAN_DIR              => 'man',
	      COLL_SUBDIR_PERMS    => undef,
              COLL_SUB_DEF_PERMS   => undef,
	      PKG_SUBDIRS          => undef,
	      PKG_ADMIN_DIR        => 'admin',
	      PKG_WWW_DIR          => 'www',
	      REQUIRED_DIRS        => undef,
	      PKG_SUBDIR_PERMS     => undef,
              PKG_SUB_DEF_PERMS    => undef,
              PKG_FILE_PERMS       => undef,
	      MAN_SUBDIRS          => undef,
	      COLL_MAN_PERMS       => undef,
	      PKG_MAN_PERMS        => undef,
	      PKG_SRC_LINK         => undef,
	      PKG_COMMON_LINK      => undef,
	      TEMPLATE_NAMES       => undef,
	      PKG_REGEXP           => undef,
	      TEMPLATE_BASE        => undef,
	      SOURCE_BASE          => undef,
	      WHATAMI_PATH         => '/bin/whatami'
	     );

# Access to the data fields as needed at run time

sub AUTOLOAD {
  my $self = shift;
  my $type = ref( $self ) || $self; 
  my $name = $AUTOLOAD;  

  $name =~ s/.*://;		# strip fully-qualified portion
  my $fields = $self->{fields};
  unless( exists $fields->{$name} ) {
      PkgDebug::error( "Can't access `$name' field" );
  }
  if( @_ ) {
      PkgDebug::warn( "Can't set the '$name' field in $type (READ ONLY)" );
  }

  return $fields->{$name};
}

###########################################################################

=head1 Public Instance Methods

=head2 new()

    IN: Optionally, the location of the pkg.cfg configuration file.
        ( call like "PkgConfig->new( $cfg_file )" )
   OUT: Returns a PkgConfig object
EFFECT: Creates and blesses a new PkgConfig object and
        initializes class fields to the values found in pkg.cfg.

=cut

sub new {
  my( $type, $cfg_file ) = @_;
  my $class = ref( $type ) || $type;
  my $self = {};

  $self->{fields} = \%fields;
  bless $self;

  $self->initialize( $cfg_file );
  return $self;
}

#########################################################################

=head2 versoin( )

    IN: 
   OUT: 
EFFECT: return version number

=cut

sub version {
    my($self) = shift;

    return $VERSION;
}

#########################################################################

=head2 initialize( $cfg_file )

    IN: Optionally, the location of the pkg.cfg config file.
   OUT: The output is not defined.
EFFECT: Reads the config file and sets fields.

=cut

sub initialize {
  my( $self, $cfg_file ) = @_;
  my $type = ref( $self ) || $self;

  PkgDebug::debug( 1, "Initializing $type object" );

  # Set the location of the config file
  if( defined $cfg_file ) {
      PkgDebug::debug( 1, "changing location of config file to $cfg_file" );
      $self->{'fields'}->{'CONFIG_LOC'} = "$cfg_file";
  }

  # Read the config file
  my( %file_hash ) = $self->_read_config_file();

  # Set SOFTWARE_ROOT and SOFTWARE_PERMS
  if( _sanity_check( $self, 'SOFTWARE_ROOT', %file_hash ) ) {
      my( @values ) = split( /:/, $file_hash{'SOFTWARE_ROOT'}, 5 );
      $self->{'fields'}->{'SOFTWARE_ROOT'} = shift( @values );
      $self->{'fields'}->{'SOFTWARE_PERMS'} = [ @values[0..2] ];
      $self->{'fields'}->{'SOFTWARE_LINK'} = $values[3];
  }

  # Set SOFT_ROOT
  if( _sanity_check( $self, 'SOFT_ROOT', %file_hash ) ) {
      $self->{'fields'}->{'SOFT_ROOT'} = $file_hash{'SOFT_ROOT'};
  }

  # Save arch perms
  my( @arch_def_perms );
  if( _sanity_check( $self, 'ARCH_DEF_PERMS', %file_hash ) ) {
      @arch_def_perms = split( /:/, $file_hash{'ARCH_DEF_PERMS'}, 3 );
      $self->{'fields'}->{'ARCH_DEF_PERMS'} = [ @arch_def_perms ];
  }

  # Set COMMON
  if( _sanity_check( $self, 'COMMON', %file_hash ) ) {
      my( @values ) = split( /:/, $file_hash{'COMMON'}, 5 );
      my( $com ) = shift( @values );
      $self->{'fields'}->{'ARCHES'} = [ "$com" ];
      $self->{'fields'}->{'COMMON'} = $com;
      if( $values[0] or $values[1] or $values[2] ) {
	  $self->{'fields'}->{'ARCH_PERMS'}->{"$com"} = [ @values[0..2] ];
      } else {
	  $self->{'fields'}->{'ARCH_PERMS'}->{"$com"} = [ @arch_def_perms ];
      }
      if($values[3]){
	  $self->{'fields'}->{'ARCH_LINKS'}->{"$com"} = $values[3];
      }
  }

  # Set ARCHES ans ARCH_PERMS
  if( _sanity_check( $self, 'ARCHES', %file_hash ) ) {
      my( $arch_ref );
      # if there is is only one architecture then ARCHES 
      # might not be an array
      if (ref ($file_hash{'ARCHES'}) eq ARRAY){
	  $arch_ref = $file_hash{'ARCHES'};
      } else {
	  $arch_ref = [$file_hash{'ARCHES'}];
      }
      my( $array_ref ) = $self->ARCHES();
      foreach( @$arch_ref ) {
	  my( @values ) = split( /:/, $_, 5 );
	  my( $arch ) = shift( @values );
	  push( @$array_ref, $arch );
	  if( $values[0] or $values[1] or $values[2] ) {
	      $self->{'fields'}->{'ARCH_PERMS'}->{"$arch"} = [ @values[0..2] ];
	  } else {
	      $self->{'fields'}->{'ARCH_PERMS'}->{"$arch"} = [ @arch_def_perms ];
	  }
	  if($values[3]){
	      $self->{'fields'}->{'ARCH_LINKS'}->{"$arch"} = $values[3];
	  }
      }
  }

  # Save collection perms
  my( @coll_def_perms );
  if( _sanity_check( $self, 'COLL_DEF_PERMS', %file_hash ) ) {
      @coll_def_perms = split( /:/, $file_hash{'COLL_DEF_PERMS'}, 3 );
      $self->{'fields'}->{'COLL_DEF_PERMS'} = [ @coll_def_perms ];
  }

  # Set COLLECTIONS and COLL_PERMS
  if( _sanity_check( $self, 'COLLECTIONS', %file_hash ) ) {
      my( $coll_ref ) = $file_hash{'COLLECTIONS'};
      my( @coll );
      foreach( @$coll_ref ) {
	  my( @values ) = split( /:/, $_, 4 );
	  my( $coll ) = shift( @values );
	  push( @coll, $coll );
	  if( $values[0] or $values[1] or $values[2] ) {
	      $self->{'fields'}->{'COLL_PERMS'}->{"$coll"} = [ @values ];
	  } else {
	      $self->{'fields'}->{'COLL_PERMS'}->{"$coll"} = [ @coll_def_perms ];
	  }
      }
      $self->{'fields'}->{'COLLECTIONS'} = [ @coll ];
  }

  # Set LOCAL_BASE
  if( _sanity_check( $self, 'LOCAL_BASE', %file_hash ) ) {
      $self->{'fields'}->{'LOCAL_BASE'} = $file_hash{'LOCAL_BASE'};
  }

  # Set COLLECT_IN_LOCAL
  if( _sanity_check( $self, 'COLLECT_IN_LOCAL', %file_hash ) ) {
      my( @coll ) = split( /:/, $file_hash{'COLLECT_IN_LOCAL'});
      $self->{'fields'}->{'COLLECT_IN_LOCAL'} = [ @coll ];
  }


  # Save coll subdir perms
  my( @coll_sub_def_perms );
  if( _sanity_check( $self, 'COLL_SUB_DEF_PERMS', %file_hash ) ) {
      @coll_sub_def_perms = split( /:/, $file_hash{'COLL_SUB_DEF_PERMS'}, 3 );
      $self->{'fields'}->{'COLL_SUB_DEF_PERMS'} = [ @coll_sub_def_perms ];
  }

  # Save pkg sub perms
  my( @pkg_sub_def_perms );
  if( _sanity_check( $self, 'PKG_SUB_DEF_PERMS', %file_hash ) ) {
      @pkg_sub_def_perms = split( /:/, $file_hash{'PKG_SUB_DEF_PERMS'}, 3 );
      $self->{'fields'}->{'PKG_SUB_DEF_PERMS'} = [ @pkg_sub_def_perms ];
  }

  # Set PACKAGE_DIR
  if( _sanity_check( $self, 'PACKAGE_DIR', %file_hash ) ) {
      my( @values ) = split( /:/, $file_hash{'PACKAGE_DIR'}, 4 );
      my( $pack ) = shift( @values );
      $self->{'fields'}->{'COLL_SUBDIRS'} = [ "$pack" ];
      $self->{'fields'}->{'PACKAGE_DIR'} = $pack;
      if( $values[0] or $values[1] or $values[2] ) {
	  $self->{'fields'}->{'COLL_SUBDIR_PERMS'}->{"$pack"} = [ @values ];
      } else {
	  $self->{'fields'}->{'COLL_SUBDIR_PERMS'}->{"$pack"} = [ @coll_sub_def_perms ];
      }
  }

  # Set MAN_DIR
  if( _sanity_check( $self, 'MAN_DIR', %file_hash ) ) {
      my( @values ) = split( /:/, $file_hash{'MAN_DIR'}, 4 );
      my( $man ) = shift( @values );
      $self->{'fields'}->{'MAN_DIR'} = $man;
      my( $array_ref ) = $self->COLL_SUBDIRS();
      push( @$array_ref, $man );
      $self->{'fields'}->{'PKG_SUBDIRS'} = [ "$man" ];
      if( $values[0] or $values[1] or $values[2] ) {
	  $self->{'fields'}->{'COLL_SUBDIR_PERMS'}->{"$man"} = [ @values ];

  $self->{'fields'}->{'PKG_SUBDIR_PERMS'}->{"$man"} = [ @values ];
      } else {
	  $self->{'fields'}->{'COLL_SUBDIR_PERMS'}->{"$man"} = [ @coll_sub_def_perms ];
	  $self->{'fields'}->{'PKG_SUBDIR_PERMS'}->{"$man"} = [ @pkg_sub_def_perms ];
      }
  }

  # Set COLL_SUBDIRS and COLL_SUBDIR_PERMS
  if( _sanity_check( $self, 'COLL_SUBDIRS', %file_hash ) ) {
      my( $coll_sub_ref ) = $file_hash{'COLL_SUBDIRS'};
      my( $array_ref ) = $self->COLL_SUBDIRS();
      foreach( @$coll_sub_ref ) {
	  my( @values ) = split( /:/, $_, 4 );
	  my( $coll_sub ) = shift( @values );
	  push( @$array_ref, $coll_sub );
	  if( $values[0] or $values[1] or $values[2] ) {
	      $self->{'fields'}->{'COLL_SUBDIR_PERMS'}->{"$coll_sub"} = [ @values ];
	  } else {
	      $self->{'fields'}->{'COLL_SUBDIR_PERMS'}->{"$coll_sub"} = [ @coll_sub_def_perms ];
	  }
      }
  }

  # Set PKG_ADMIN_DIR
  if( _sanity_check( $self, 'PKG_ADMIN_DIR', %file_hash ) ) {
      my( @values ) = split( /:/, $file_hash{'PKG_ADMIN_DIR'}, 4 );
      my( $admin ) = shift( @values );
      my( $array_ref ) = $self->PKG_SUBDIRS();
      push( @$array_ref, $admin );
      $self->{'fields'}->{'PKG_ADMIN_DIR'} = $admin;
      if( $values[0] or $values[1] or $values[2] ) {
	  $self->{'fields'}->{'PKG_SUBDIR_PERMS'}->{"$admin"} = [ @values ];
      } else {
	  $self->{'fields'}->{'PKG_SUBDIR_PERMS'}->{"$admin"} = [ @pkg_sub_def_perms ];
      }
  }

  # Set PKG_WWW_DIR
  if( _sanity_check( $self, 'PKG_WWW_DIR', %file_hash ) ) {
      my( @values ) = split( /:/, $file_hash{'PKG_WWW_DIR'}, 4 );
      my( $www ) = shift( @values );
      my( $array_ref ) = $self->PKG_SUBDIRS();
      push( @$array_ref, $www );
      $self->{'fields'}->{'PKG_WWW_DIR'} = $www;
      if( $values[0] or $values[1] or $values[2] ) {
	  $self->{'fields'}->{'PKG_SUBDIR_PERMS'}->{"$www"} = [ @values ];
      } else {
	  $self->{'fields'}->{'PKG_SUBDIR_PERMS'}->{"$www"} = [ @pkg_sub_def_perms ];
      }
  }

  # Set PKG_SUBDIRS and PKG_SUBDIR_PERMS
  if( _sanity_check( $self, 'PKG_SUBDIRS', %file_hash ) ) {
      my( $pkg_sub_ref ) = $file_hash{'PKG_SUBDIRS'};
      my( $array_ref ) = $self->PKG_SUBDIRS();
      foreach( @$pkg_sub_ref ) {
	  my( @values ) = split( /:/, $_, 4 );
	  my( $pkg_sub ) = shift( @values );
	  push( @$array_ref, $pkg_sub );
	  if( $values[0] or $values[1] or $values[2] ) {
	      $self->{'fields'}->{'PKG_SUBDIR_PERMS'}->{"$pkg_sub"} = [ @values ];
	  } else {
	      $self->{'fields'}->{'PKG_SUBDIR_PERMS'}->{"$pkg_sub"} = [ @pkg_sub_def_perms ];
	  }
      }
  }

  # Set REQUIRED_DIRS
  if( _sanity_check( $self, 'REQUIRED_DIRS', %file_hash ) ) {
      my( $required_ref ) = $file_hash{'REQUIRED_DIRS'};
      $self->{'fields'}->{'REQUIRED_DIRS'} = $required_ref;
  }

  # Set PKG_FILE_PERMS
  if( _sanity_check( $self, 'PKG_FILE_PERMS', %file_hash ) ) {
      my @pkg_file_perms = split( /:/, $file_hash{'PKG_FILE_PERMS'}, 3 );
      $self->{'fields'}->{'PKG_FILE_PERMS'} = [ @pkg_file_perms ];
  }

  # Set MAN_SUBDIRS, COLL_MAN_PERMS, and PKG_MAN_PERMS
  if( _sanity_check( $self, 'MAN_SUBDIRS', %file_hash ) ) {
      my( $man_sub_ref ) = $file_hash{'MAN_SUBDIRS'};
      my( @man );
      foreach( @$man_sub_ref ) {
	  my( @values ) = split( /:/, $_, 4 );
	  my( $man_sub ) = shift( @values );
	  push( @man, $man_sub );
	  if( $values[0] or $values[1] or $values[2] ) {
	      $self->{'fields'}->{'COLL_MAN_PERMS'}->{"$man_sub"} = [ @values ];
	      $self->{'fields'}->{'PKG_MAN_PERMS'}->{"$man_sub"} = [ @values ];
	  } else {
	      $self->{'fields'}->{'COLL_MAN_PERMS'}->{"$man_sub"} = [ @coll_sub_def_perms ];
	      $self->{'fields'}->{'PKG_MAN_PERMS'}->{"$man_sub"} = [ @pkg_sub_def_perms ];
	  }
      }
      $self->{'fields'}->{'MAN_SUBDIRS'} = [ @man ];
  }

  # Set PKG_SRC_LINK
  if( _sanity_check( $self, 'PKG_SRC_LINK', %file_hash ) ) {
      $self->{'fields'}->{'PKG_SRC_LINK'} = $file_hash{'PKG_SRC_LINK'};
  }

  # Set PKG_COMMON_LINK
  if( _sanity_check( $self, 'PKG_COMMON_LINK', %file_hash ) ) {
      $self->{'fields'}->{'PKG_COMMON_LINK'} = $file_hash{'PKG_COMMON_LINK'};
  }

  # Set TEMPLATE_NAMES
  if( _sanity_check( $self, 'TEMPLATE_NAMES', %file_hash ) ) {
      my( $temp_ref ) = $file_hash{'TEMPLATE_NAMES'};
      foreach( @$temp_ref ) {
	  my( $temp, $name, $dir, $loc ) = split( /:/, $_, 4 );
	  if ($loc =~ /^%/) {
	      $loc =~ s/^%//;
	      $loc = $self->{'fields'}->{$loc};
	  }
	  $self->{'fields'}->{'TEMPLATE_NAMES'}->{"$temp"} = [$name, $dir, $loc];
      }
  }

  # Set PKG_REGEXP
  if( _sanity_check( $self, 'PKG_REGEXP', %file_hash ) ) {
      $self->{'fields'}->{'PKG_REGEXP'} = $file_hash{'PKG_REGEXP'};
  }

  # Set TEMPLATE_BASE
  if( _sanity_check( $self, 'TEMPLATE_BASE', %file_hash ) ) {
      $self->{'fields'}->{'TEMPLATE_BASE'} = $file_hash{'TEMPLATE_BASE'};
  }

  # Set SOURCE_BASE
  if( _sanity_check( $self, 'SOURCE_BASE', %file_hash ) ) {
      $self->{'fields'}->{'SOURCE_BASE'} = $file_hash{'SOURCE_BASE'};
  }

  # Set WHATAMI_PATH
  if( _sanity_check( $self, 'WHATAMI_PATH', %file_hash ) ) {
      $self->{'fields'}->{'WHATAMI_PATH'} = $file_hash{'WHATAMI_PATH'};
  }

  PkgDebug::debug( 1, "Object $type initialized" );
}

###########################################################################

=head2 _read_config_file()

    IN: There are no input parameters.
   OUT: Returns a hashed version of the pkg.cfg file with the keys set to
        the fields in the file, and the values set to the data values
        specified in the cfg file. If more than one value is specified
        for a field, the value will be an array reference. Lines that
        begin with a "#" are considered comments.
EFFECT: There are no side effects.

=cut

sub _read_config_file {
    my( $self ) = shift;
    my( $cfg_file ) = $self->CONFIG_LOC();
    my( %file_hash );
    my( $current );

    PkgDebug::debug( 6, "Reading $cfg_file" );

    open( CFG, "<$cfg_file" ) || PkgDebug::error( "Can't read $cfg_file" );

    while( <CFG> ) {
	my( $line ) = $_;
	chop( $line );
	$line =~ s/\s//g;

	next if( $line eq "" || /^\#/ );

	if( $line =~ /^%%(.*)$/ ) {
	    $current = $1;
	    next;
	}
	else {
	    unless( $file_hash{$current} ) {
		$file_hash{$current} = "$line";
	    }
	    else {
		my( $old ) = $file_hash{$current};
		if( ref $old ) {
		    push( @$old, "$line" );
		}
		else {
		    $file_hash{$current} = [ "$old", "$line" ];
		}
	    }
	}
    }

    close( CFG );

    return( %file_hash );
}

###########################################################################

=head2 _sanity_check()

    IN: The key to check for in the config file and the file hash.
   OUT: True if the key is in the config file, False otherwise.
EFFECT: Checks the validity of the wanted key to make sure it exists.

=cut

sub _sanity_check {
  my( $self, $entry, %parsed_hash ) = @_;

  unless( defined $parsed_hash{$entry} ) {
      if( defined $self->{'fields'}->{$entry} ) {
	  PkgDebug::warn( "Entry $entry is not defined in pkg.cfg, using default" );
	  return( 0 );
      }
      else {
	  PkgDebug::error( "Entry $entry is not defined in pkg.cfg" );
      }
  }

  return( 1 );
}

###########################################################################
