=head1 NAME PkgSystem.pm

PgkSystem.pm - The PkgSystem module makes all system calls for the pkg
project. The module has the ability to print out the commands it would execute
in standard sh shell script to STDOUT if given a dry-run flag.

$Id: PkgSystem.pm,v 1.41 1999/12/20 20:11:58 gomberg Exp $

=cut

my($VERSION) = "PkgSystem " . "$Revision: 1.41 $foo";

package    PkgSystem;
use        PkgDebug;
use        File::Copy;
use        Cwd;
use        Cwd 'chdir';

########################################################################

=head1 Class Fields

The PkgSystem object has two private class fields.

  PKG_CONFIG_REF is a refrence to the config object and is set
                 during object creation.
  DRY_RUN        is set to true when system commands should be
                 printed out and not executed.

=cut

my %fields = ( PKG_CONFIG_REF => undef,
               DRY_RUN        => undef
              );

########################################################################

=head1 Public Instance Methods

=head2 new()

    IN: A refrence to a PkgConfig object (call like "PkgSystem->new($cfg_ref)")
   OUT: Returns a PkgSystem object
EFFECT: Creates and blesses a new PkgSystem object and
        initializes class fields (to zero).

=cut

sub new {
    my( $type, $cfg_ref ) = @_;
    my $class = ref( $type ) || $type;
    my $self = {};

    $self->{'fields'} = \%fields;
    bless $self, $class;

    if( ref( $cfg_ref ) eq "PkgConfig" ) {
	$self->{'fields'}->{'PKG_CONFIG_REF'} = $cfg_ref;
    } else {
        PkgDebug::error( "need to create with a PkgConfig refrence" );
    }

    $self->_initialize();
    return $self;
}

########################################################################

=head2 version()

    IN: 
   OUT: 
EFFECT: return version number

=cut

sub version {
    my($self) = shift;

    return $VERSION;
}

########################################################################

=head2 set_dry_run( $value )

    IN: The $value to set the DRY_RUN field to.
   OUT: The current value of the DRY_RUN field.
EFFECT: If $value is specified, the DRY_RUN field is set to it. Any
        value > 0 will cause system commands to be printed to STDOUT
        in sh shell script instead of being executed.

=cut

sub set_dry_run {
    my( $self, $value ) = @_;
    my $type = ref( $self ) || $self;

    if( defined $value and $value ne '' ) {
	PkgDebug::debug( 3, "Setting DRY_RUN field in $type to $value" );
	$self->{'fields'}->{'DRY_RUN'} = $value;
    }
    return( $self->{'fields'}->{'DRY_RUN'} );
}

########################################################################

=head2 make_dir( $directory, $group, $permissions )

    IN: A fully qualified directory name, the group to set the directory
        to, and the permissions to give the directory.
   OUT: True if directory was created, False otherwise.
EFFECT: Creates specified directory with group and permissions. The owner
        will be the person running the script.

=cut

sub make_dir {
    my( $self, $dir, $group, $perms ) = @_;
    
    PkgDebug::debug( 9, "Attempting to create $dir" );

    unless( $dir and $group and $perms ) {
	PkgDebug::error( "make_dir failed. Expecting a dir name, group and perms" );
    }

    if( -e $dir ) {
	PkgDebug::warn( "make_dir failed. File $dir already exists" );
	return 0;
    }

    if( $self->{'fields'}->{'DRY_RUN'} ) {
	print STDOUT "mkdir $dir\n";
    } else {
	# make the directory, but the permissions aren't always set
	# correctly by perl for some unknown reason, so go with a 
	# paranoid default and pass the perms to the change_mod function
	#
	unless( mkdir( $dir, 0700 ) ) {
	  PkgDebug::warn( "make_dir failed. Dir $dir not created." );
	  return 0;
	}
    }

    unless( $self->change_grp( $group, $dir ) and
	    $self->change_mod( $perms, $dir ) ) {
	return 0;
    }

    return 1;
}

########################################################################

=head2 remove_dir( $directory, $recursive_flag )

    IN: A fully qualified directory name. A recursive flag
   OUT: True if directory was removed, False otherwise.
EFFECT: Removes specified directory from the system. If the recursive flag
    is present the remove will remove any subdirectories as well as the directory.
    The directory and subdirectories all have to be empty.

=cut

sub remove_dir {
    my( $self, $dir, $recursive_flag ) = @_;

    PkgDebug::debug( 9, "Attempting to remove $dir" );

    unless( $dir ) {
	PkgDebug::error( "remove_dir failed. Expecting a directory name" );
    }

    if( $self->{'fields'}->{'DRY_RUN'} ) {
	print STDOUT "rmdir $dir\n";
    }
    else {
	if(defined($recursive_flag) && $recursive_flag == 1){
	    my(@dir_list);
	    my($sub_dir);

	    @dir_list = $self->get_subdirs($dir);
	    foreach $sub_dir (@dir_list){
		$self->remove_dir($sub_dir, 1);
	    }			
	}

	# return if the directory isn't empty
	return if(!$self->is_empty($dir));
	unless( -e $dir ) {
	    PkgDebug::warn( "remove_dir failed. Dir $dir does not exist" );
	    return 0;
	}

	unless( rmdir $dir ) {
	    PkgDebug::warn( "remove_dir failed. Dir $dir not removed" );
	    return 0;
	}


    }

    return 1;
}

########################################################################

=head2 make_link( $dest, $src )

    IN: The file to be linked, and the location to link it to.
        I.E. make_link( A, B ) will create B -> A
   OUT: True if link was created, False otherwise.
EFFECT: Creates a symbolic link in the file system.

=cut

sub make_link {
    my( $self, $dest, $src ) = @_;

    PkgDebug::debug( 9, "Attempting to link $src to $dest" );

    unless( $dest and $src ) {
	PkgDebug::error( "make_link failed. Need to define a dest and src" );
    }

    if( -e $src ) {
	PkgDebug::warn( "make_link failed. File $src already exists" );
	return 0;
    }

    if( $self->{'fields'}->{'DRY_RUN'} ) {
	print STDOUT "ln -s $dest $src\n";
    }
    else {
	my( $check_dest ) = $dest;
	# For relative destinations, let the file system figure it out
	if( $dest =~ /^\./ ) {
	    my( $check_src ) = $src;
	    $check_src =~ s:\/[\w\.-]+$::;
	    $check_dest = $check_src . '/' . $dest;
	}
	unless( -e $check_dest ) {
	    PkgDebug::warn( "make_link failed. File $check_dest does not exist" );
	    return 0;
	}

	unless( symlink $dest, $src ) {
	    PkgDebug::warn( "make_link failed. Link not created" );
	    return 0;
	}
    }

    return 1;
}

########################################################################

=head2 remove_link( $link )

    IN: The name of the symbolic link to remove.
   OUT: True if the link was removed, False otherwise.
EFFECT: The named sym-link is removed for the file system.

=cut

sub remove_link {
    my( $self, $link ) = @_;

    PkgDebug::debug( 9, "Attempting to unlink $link" );

    unless( $link ) {
	PkgDebug::error( "remove_link failed. Need to specify link to remove" );
    }

    if( $self->{'fields'}->{'DRY_RUN'} ) {
	print STDOUT "rm $link\n";
    }
    else {
	unless( -l $link ) {
	    PkgDebug::warn( "remove_link failed. $link is not a symbolic link" );
	    return 0;
	}

	unless( unlink $link ) {
	    PkgDebug::warn( "remove_link failed. $link not removed" );
	    return 0;
	}
    }

    return 1;
}

########################################################################

=head2 remove_file( $file )

    IN: A filename.
   OUT: True if the file was removed, False otherwise.
EFFECT: Removes specified file from the file system.

=cut

sub remove_file {
    my( $self, $file ) = @_;

    PkgDebug::debug( 9, "Attempting to remove $file" );

    unless( $file ) {
	PkgDebug::error( "remove_file failed. File name expected" );
    }

    if( $self->{'fields'}->{'DRY_RUN'} ) {
	print STDOUT "rm $file\n";
    }
    else {
	unless( -f $file ) {
	    PkgDebug::warn( "remove_file failed. $file does not exist" );
	    return 0;
	}

	unless( unlink $file ) {
	    PkgDebug::warn( "remove_file failed. $file not removed" );
	    return 0;
	}
    }

    return 1;
}

########################################################################

=head2 copy_file( $src, $dest )

    IN: A file or directory name (src), and a location to
        copy that file or directory to (dest).
   OUT: True if copy was successful, False otherwise.
EFFECT: Make a copy of src at dest.

=cut

sub copy_file {
    my( $self, $src, $dest ) = @_;

    PkgDebug::debug( 9, "Attempting to copy $src to $dest" );

    unless( $dest and $src ) {
	PkgDebug::error( "copy_file failed. Need to define a src and dest" );
    }

    if( -e $dest ) {
	PkgDebug::warn( "copy_file failed. File $dest already exists" );
	return 0;
    }

    if( $self->{'fields'}->{'DRY_RUN'} ) {
	print STDOUT "cp $src $dest\n";
    }
    else {
	unless( -e $src ) {
	    PkgDebug::warn( "copy_file failed. File $src does not exist" );
	    return 0;
	}

	unless( copy( $src, $dest ) ) {
	    PkgDebug::warn( "copy_file failed. File not copied" );
	    return 0;
	}
    }

    return 1;
}

########################################################################

=head2 rename_file( $orig, $new )

    IN: A file or directory name (orig), and a name (new) to
        rename that file or directory to.
   OUT: True if rename was successful, False otherwise.
EFFECT: Make a copy of orig to new and remove orig.

=cut

sub rename_file {
    my( $self, $orig, $new ) = @_;

    PkgDebug::debug( 9, "Attempting to rename $orig to $new" );

    if( $self->copy_file( $orig, $new ) ) {
	if( $self->remove_file( $orig ) ) {
	    return 1;
	}
	return 0;
    }

    return 0;
}

########################################################################

=head2 change_mod( $perms, $file )

    IN: A file name (or directory) and the access permissions to set.
   OUT: True if the access permissions were changed, False otherwise.
EFFECT: Change file (or directory) access permissions.

=cut

sub change_mod {
    my( $self, $perms, $file ) = @_;

    PkgDebug::debug( 9, "Changing permissions on $file" );

    unless( $perms and $file ) {
	PkgDebug::error( "change_mod failed. Need to define perms and file" );
    }

    if( $perms !~ /^\d+$/ ) {
        PkgDebug::warn( "change_mod failed. $perms are not valid perms" );
	return 0;
    }
    # Let's really make it look octal.
    if( $perms !~ /^0/) {
	$perms = '0' . $perms;
    }
    

    if( $self->{'fields'}->{'DRY_RUN'} ) {
	print STDOUT "chmod $perms $file\n";
    }
    else {
	unless( -e $file ) {
	    PkgDebug::warn( "change_mod failed. $file does not exist" );
	    return 0;
	}

	unless( chmod( oct $perms, $file ) ) {
	    PkgDebug::warn( "change_mod failed. $file not altered" );
	    return 0;
	}
    }

    return 1;
}

########################################################################

=head2 change_own( $owner, $file )

    IN: A user name and a file name (or directory).
   OUT: True if the file owner is changed, False otherwise.
EFFECT: Translates $owner into a system UID and changes the ownership
        of the specified file (or directory) to that UID.

=cut

sub change_own {
    my( $self, $owner, $file ) = @_;

    PkgDebug::debug( 9, "Changing ownership on $file" );

    unless( $owner and $file ) {
	PkgDebug::error( "change_own failed. Need to define owner and file" );
    }

    my $uid = (getpwnam( $owner ))[2];

    unless( $uid ) {
	PkgDebug::warn( "change_own failed. Can't get $owner UID" );
	return 0;
    }
    
    if( $self->{'fields'}->{'DRY_RUN'} ) {
	print STDOUT "chown $owner $file\n";
    }
    else {
	unless( -e $file ) {
	    PkgDebug::warn( "change_own failed. $file does not exist" );
	    return 0;
	}

	my $perms = (stat( $file ))[2];
	$perms = sprintf( "%lo", $perms );

	unless( chown $uid, '-1', $file ) {
	    PkgDebug::warn( "change_own failed. $file not altered" );
	    return 0;
	}

	# Perl's chown has a nasty habit of killing the sticky bits
	unless( $self->change_mod( $perms, $file ) ) {
	    return 0;
	}

    }

    return 1;
}

########################################################################

=head2 change_grp( $group, $file )

    IN: A group name and a file name (or directory).
   OUT: True if the file group is changed, False otherwise.
EFFECT: Translates $group into a system UID and changes the ownership
        of the specified file (or directory) to that UID.

=cut

sub change_grp {
    my( $self, $group, $file ) = @_;

    PkgDebug::debug( 9, "Changing group on $file" );

    unless( $group and $file ) {
	PkgDebug::error( "change_grp failed. Need to define group and file" );
    }

    my $gid = (getgrnam( $group ))[2];
    my $uid = (stat( $file ))[4];

    unless( $gid ) {
	PkgDebug::warn( "change_grp failed. Can't get $group GID" );
	return 0;
    }
    
    if( $self->{'fields'}->{'DRY_RUN'} ) {
	print STDOUT "chgrp $group $file\n";
    }
    else {
	unless( -e $file ) {
	    PkgDebug::warn( "change_grp failed. $file does not exist" );
	    return 0;
	}

	my $perms = (stat( $file ))[2];
	$perms = sprintf( "%lo", $perms );

	unless( chown $uid, $gid, $file ) {
	    PkgDebug::warn( "change_grp failed. $file not altered" );
	    return 0;
	}

	# Perl's chown has a nasty habit of killing the sticky bits
	unless( $self->change_mod( $perms, $file ) ) {
	    return 0;
	}

    }

    return 1;
}

########################################################################

=head2 get_cwd()

    IN: There are no input arguments
   OUT: Returns the fully qualified current working directory or
        "undefined" if cwd can not be determined.
EFFECT: No side effects.

=cut

sub get_cwd {
    my $self = shift;

    PkgDebug::debug( 9, "Getting current working directory" );

    my $cwd_dir = cwd();

    unless( $cwd_dir ) {
	PkgDebug::warn( "get_cwd failed." );
	return undef;
    } else {
	return $cwd_dir;
    }
}

########################################################################

=head2 set_cwd()

    IN: Directory to set the current working directory to.
   OUT: True if the directory is changed, False otherwise.
EFFECT: Updates $ENV{PWD}.

=cut

sub set_cwd {
    my( $self, $dir ) = @_;

    PkgDebug::debug( 9, "Setting current working directory" );

    if( chdir( $dir ) ) {
	return( 1 );
    } else {
        PkgDebug::warn( "Can't change dir to $dir" );
	return( 0 );
    }
}

########################################################################

=head2 get_whatami()

    IN: There are no input arguments
   OUT: Returns the current architecture whatami string or
        "undefined" if whatami string is not availiable.
EFFECT: No side effects.

=cut

sub get_whatami {
    my $self = shift;

    PkgDebug::debug( 9, "Getting current whatami string" );

    my $whatami = $self->{'fields'}->{'PKG_CONFIG_REF'}->WHATAMI_PATH();
    my $what = `$whatami`;
    chop( $what );

    unless( $what ) {
	PkgDebug::warn( "get_whatami failed. Can't run $whatami?" );
	return undef;
    } else {
	return $what;
    }
}

########################################################################

=head2 exists( $file )

    IN: A file name to be checked.
   OUT: One of the following integers
        0 - File does not exist
        1 - File is a standard file
        2 - File is a symbolic link
        3 - File is a directory
        4 - File exists but is of an unknown type
EFFECT: No side effects.

=cut

sub exists {
    my( $self, $file ) = @_;
    my $type = ref( $self ) || $self;

    PkgDebug::debug( 9, "Checking if $file exists" );

    unless( $file ) {
	PkgDebug::error( "exists failed. Need to specify a file" );
	return 0;
    }

    return 0 unless( -e $file );

    return 1 if( -f $file );

    return 2 if( -l $file );

    return 3 if( -d $file );

    return 4;
}

########################################################################

=head2 get_files( $directory )

    IN: A directory name.
   OUT: An array of all fully qualified file names (not subdirectories
        or links) in the given directory. Returns undef if an error
        occurs or the directory contains no files.
EFFECT: No side effects

=cut

sub get_files {
    my ( $self, $dir ) = @_;

    PkgDebug::debug( 9, "Getting files in $dir" );

    unless( -d $dir ) {
	PkgDebug::warn( "get_files failed. $dir is not a valid directory" );
	return undef;
    }

    unless( opendir( DIR, "$dir" ) ) {
	PkgDebug::warn( "get_files failed. Can't open $dir" );
	return undef;
    }

    $dir .= '/' if( $dir !~ /\/$/ );

    my @files = grep !-l, grep -f, map "$dir$_", readdir DIR;
    
    closedir DIR;

    # protect against broken readdirs
    my (@return_files);
    foreach (@files) {
	push (@return_files, $_) if -e $_;
    }

    return @return_files;
}

########################################################################

=head2 get_subdirs( $directory, $short_path )

    IN: A directory name, a switch that specifies if full pathnames or just 
        directory names are returned.
   OUT: An array of all fully qualified subdirectory names (not files
        or links) in the given directory. Returns undef if an error
        occurs or the directory contains no subdirs.
EFFECT: No side effects

=cut

sub get_subdirs {
    my ( $self, $dir, $short_path ) = @_;

    PkgDebug::debug( 9, "Getting subdirectories in $dir" );

    unless( -d $dir ) {
	PkgDebug::warn( "get_subdirs failed. $dir is not a valid directory" );
	return undef;
    }

    unless( opendir( DIR, "$dir" ) ) {
	PkgDebug::warn( "get_subdirs failed. Can't open $dir" );
	return undef;
    }

    $dir .= '/' if( $dir !~ /\/$/ );

    my @dirs = readdir DIR;
    
    # remove the dots and find just the subdirs in the file list
    my @subdirs_orig = grep (-d && !-l, (map "$dir$_", (grep (!/^\.{1,2}$/, @dirs))));

    closedir DIR;

    # protect against broken readdirs
    my (@subdirs);
    foreach (@subdirs_orig) {
	push (@subdirs, $_) if -e $_;
    }

    if(defined($short_path) && $short_path){
	my @subdirs_short = grep (s/^.*\/([^\/]+)$/$1/, @subdirs);
	return @subdirs_short;
    }

    return @subdirs;
}

########################################################################

=head2 get_links( $directory )

    IN: A directory name.
   OUT: A hash of all fully qualified symbolic links in the given
        directory (hash keys) and the destination that they point
        to (hash values). For example:
            $link_hash{'/tmp/linkfile'} = '/usr/lib/somefile'
        Means that the sym-link "linkfile" points to "somefile". Be
        warned that due to the perl implementation of readlink, some
        some of the hash values may not be fully qualified.        
EFFECT: No side effects

=cut

sub get_links {
    my ( $self, $dir ) = @_;

    PkgDebug::debug( 9, "Getting symlinks in $dir" );

    unless( -d $dir ) {
	PkgDebug::warn( "get_links failed. $dir is not a valid directory" );
	return undef;
    }

    unless( opendir( DIR, "$dir" ) ) {
	PkgDebug::warn( "get_links failed. Can't open $dir" );
	return undef;
    }

    $dir .= '/' if( $dir !~ /\/$/ );

    my @links = grep -l, map "$dir$_", readdir DIR;

    closedir DIR;

    my %links;

    foreach( @links ) {
	$links{"$_"} = readlink;
    }

    return %links;
}

########################################################################

=head2 is_empty( $dir, $recursive )

    IN: A directory name, recursive flag 
   OUT: True if directory is empty or recursively if this directory 
        and subdirs are empty, False otherwise. Will return
        False if an error occurs (i.e. dir does not exist).
EFFECT: No side effects

=cut

sub is_empty {
    my( $self, $dir, $recursive ) = @_;

    return 0 unless( $self->exists( $dir ) );

    my( @files ) = $self->get_files( $dir );
    return 0 if( @files );

    my( %links ) = $self->get_links( $dir );
    return 0 if( keys %links );

    my( @subdirs ) = $self->get_subdirs( $dir );
    if( @subdirs && defined($recursive) && $recursive) {
	my($subdir);
	foreach $subdir (@subdirs){
	    return 0 if(!$self->is_empty($subdir, 1));
	}
    }

    return 1;
}

########################################################################

=head1 Private Class Methods

=head2 _initialize()

    IN: There are no input arguments.
   OUT: The return value is not defined.
EFFECT: Initalizes class fields to default values.

=cut

sub _initialize {
    my $self = shift;
    my $type = ref( $self ) || $self;

    PkgDebug::debug( 1, "Creating and initializing $type object" );

    $self->{'fields'}->{'DRY_RUN'} = 0;
}

########################################################################
